import { render } from 'lit-html';
function reRender(ctx) {
    if (ctx.isConnected) {
        if (ctx.__wait) {
        }
        else {
            ctx.__wait = true;
            Promise.resolve().then(() => {
                ctx.render();
                ctx.__wait = false;
            });
        }
    }
}
export function property() {
    return function reg(_class, prop) {
        Object.defineProperty(_class, prop, {
            get: function () {
                return this['_' + prop];
            },
            set: function (x) {
                const oldValue = this['_' + prop];
                this['_' + prop] = x;
                if (this.valuesChanged) {
                    this.valuesChanged('property', prop, oldValue, x);
                }
                reRender(this);
            }
        });
    };
}
export function attribute() {
    return function reg(_class, prop) {
        Object.defineProperty(_class, prop, {
            get: function () {
                return this['_' + prop];
            },
            set: function (x) {
                const oldValue = this['_' + prop];
                this['_' + prop] = x;
                if (this.valuesChanged) {
                    this.valuesChanged('property', prop, oldValue, x);
                }
                reRender(this);
            }
        });
        const attribute = prop
            .replace(/([a-z])([A-Z])/g, '$1-$2')
            .replace(/\s+/g, '-')
            .toLowerCase();
        if (!_class['_observedAttributesMap']) {
            _class['_observedAttributesMap'] = new Map();
        }
        _class['_observedAttributesMap'].set(attribute, prop);
        if (_class['_observedAttributes']) {
            _class['_observedAttributes'].push(attribute);
        }
        else {
            _class['_observedAttributes'] = [];
            _class['_observedAttributes'].push(attribute);
        }
    };
}
export function customElement(elementName, extended) {
    return function reg(elementClass) {
        if (!customElements.get(elementName)) {
            const rendermethod = elementClass.prototype.render;
            const connectedCallbackMethod = elementClass.prototype.connectedCallback;
            const disconnectedCallbackMethod = elementClass.prototype.disconnectedCallback;
            const attributeChangedCallbackMethod = elementClass.prototype.attributeChangedCallback;
            const valuesChangedMethod = elementClass.prototype.valuesChanged;
            Object.defineProperty(elementClass, 'observedAttributes', {
                get: function () {
                    return elementClass.prototype['_observedAttributes'];
                }
            });
            elementClass.prototype.render = function (...result) {
                render(rendermethod.call(this, ...result), this, { eventContext: this });
            };
            elementClass.prototype.connectedCallback = function () {
                if (connectedCallbackMethod) {
                    connectedCallbackMethod.call(this);
                }
                this.render();
            };
            elementClass.prototype.disconnectedCallback = function () {
                if (disconnectedCallbackMethod) {
                    disconnectedCallbackMethod.call(this);
                }
            };
            elementClass.prototype.attributeChangedCallback = function (name, oldValue, newValue) {
                const nameProp = this['_observedAttributesMap'].get(name);
                this[nameProp] = newValue || '';
                if (attributeChangedCallbackMethod) {
                    attributeChangedCallbackMethod.call(this, name, oldValue, newValue);
                }
                if (valuesChangedMethod) {
                    valuesChangedMethod('attribute', name, oldValue, newValue);
                }
                reRender(this);
            };
            if (extended) {
                customElements.define(elementName, elementClass, extended);
            }
            else {
                customElements.define(elementName, elementClass);
            }
        }
        else {
        }
    };
}
//# sourceMappingURL=decorators.js.map