import { instance } from './instance';

class EventAggregator {
    private channels: any;

    constructor() {
        this.channels = {};
    }

    // microtask
    public publish(channel: string, ...args: any[]): void {
        Promise.resolve().then(() => {
            if (Array.isArray(this.channels[channel])) {
                for (let i = 0, len = this.channels[channel].length; i < len; i++) {
                    this.channels[channel][i].func.apply(this, args);
                }
            } else {
                // if no one is listening then do we really need to do anything ?
            }
        });
    }

    // sync
    public unSubscribe(channel: string, ctx: any): void {
        if (Array.isArray(this.channels[channel])) {
            for (let i = 0, len = this.channels[channel].length; i < len; i++) {
                if (this.channels[channel][i].ctx === ctx) {
                    this.channels[channel].splice(i, 1);
                }
            }
        } else {
            // channel does not exist, great!
        }
    }

    // sync
    public subscribe(channel: string, ctx: any, func: Function): void {
        if (!Array.isArray(this.channels[channel])) {
            this.channels[channel] = [];
        }
        this.channels[channel].push({ ctx: ctx, func: func });
    }
}

export function publish(channel: string, ...args: any[]) {
    instance(EventAggregator).publish(channel, ...args);
}

export function unSubscribe(channel: string, ctx: any) {
    instance(EventAggregator).unSubscribe(channel, ctx);
}

export function subscribe(channel: string, ctx: any, func: Function) {
    instance(EventAggregator).subscribe(channel, ctx, func);
}
