import { render } from 'lit-html';

function reRender(ctx: any) {
    if (ctx.isConnected) {
        if (ctx.__wait) {
        } else {
            ctx.__wait = true;
            Promise.resolve().then(() => {
                ctx.render();
                ctx.__wait = false;
            });
        }
    }
}

export function property(): Function {
    return function reg(_class: Function, prop: string): void {
        Object.defineProperty(_class, prop, {
            get: function() {
                return this['_' + prop];
            },
            set: function(x: any) {
                const oldValue = this['_' + prop];
                this['_' + prop] = x;
                if (this.valuesChanged) {
                    this.valuesChanged('property', prop, oldValue, x);
                }
                reRender(this);
            }
        });
    };
}

/**
 * @attibute- for
 */
export function attribute(): Function {
    return function reg(_class: Function, prop: string): void {
        Object.defineProperty(_class, prop, {
            get: function() {
                return this['_' + prop];
            },
            set: function(x: any) {
                const oldValue = this['_' + prop];
                this['_' + prop] = x;
                if (this.valuesChanged) {
                    this.valuesChanged('property', prop, oldValue, x);
                }
                reRender(this);
            }
        });
        // replace uppercase with lower and add '-'
        const attribute = prop
            .replace(/([a-z])([A-Z])/g, '$1-$2')
            .replace(/\s+/g, '-')
            .toLowerCase();

        //create a map so we can find it later
        if (!_class['_observedAttributesMap']) {
            _class['_observedAttributesMap'] = new Map();
        }
        _class['_observedAttributesMap'].set(attribute, prop);

        // add to observedAttributes
        if (_class['_observedAttributes']) {
            _class['_observedAttributes'].push(attribute);
        } else {
            _class['_observedAttributes'] = [];
            _class['_observedAttributes'].push(attribute);
        }
    };
}

export function customElement(elementName: string, extended?: ElementDefinitionOptions) {
    return function reg(elementClass: Function) {
        /**
         * first of all check if custom element is defined
         */
        if (!customElements.get(elementName)) {
            /**
             * get methods
             */
            const rendermethod = elementClass.prototype.render;
            const connectedCallbackMethod = elementClass.prototype.connectedCallback;
            const disconnectedCallbackMethod = elementClass.prototype.disconnectedCallback;
            const attributeChangedCallbackMethod = elementClass.prototype.attributeChangedCallback;
            const valuesChangedMethod = elementClass.prototype.valuesChanged;

            /**
             * observedAttributes
             */
            Object.defineProperty(elementClass, 'observedAttributes', {
                get: function() {
                    return elementClass.prototype['_observedAttributes'];
                }
            });

            /**
             * render
             */
            elementClass.prototype.render = function(...result: any[]) {
                render(rendermethod.call(this, ...result), this, { eventContext: this });
            };

            /**
             * connectedCallback
             */
            elementClass.prototype.connectedCallback = function() {
                if (connectedCallbackMethod) {
                    connectedCallbackMethod.call(this);
                }

                this.render();
            };

            /**
             * disconnectedCallback
             */
            elementClass.prototype.disconnectedCallback = function() {
                if (disconnectedCallbackMethod) {
                    disconnectedCallbackMethod.call(this);
                }
            };

            /**
             * attributeChangedCallback & valuesChangedMethod
             */
            elementClass.prototype.attributeChangedCallback = function(name: string, oldValue: string, newValue: string) {
                //get map
                const nameProp = this['_observedAttributesMap'].get(name);
                this[nameProp] = newValue || '';

                // if normal attributeChanged is set
                if (attributeChangedCallbackMethod) {
                    attributeChangedCallbackMethod.call(this, name, oldValue, newValue);
                }

                //if our simpler method is set
                if (valuesChangedMethod) {
                    valuesChangedMethod('attribute', name, oldValue, newValue);
                }
                reRender(this);
            };

            if (extended) {
                customElements.define(elementName, elementClass, extended);
            } else {
                customElements.define(elementName, elementClass);
            }
        } else {
            // console.log('element already registered');
        }
    };
}
